import sys
import getopt
import csv
import os
import struct

signal_count = 0

class Signal:
    'class for signals located in csv files'

    def get_field(self,header,line,field):
        found = False
        prev_line_num = -1
        while (line != '</signal>') and (found == False):   #skip to filepostscript
            if line == field:
                found = True
                break
            line = header.readline().strip()
        if found == False:
            print('Error: No ', field, ' field found in <signal>')
            sys.exit(1)
        out = header.readline().strip()
        return out

    def scale_sample(self,sample):
        if sample >= 0:
            self.scaled_sample = int((self.digital_max * sample)/self.physical_max)
        else:
            self.scaled_sample = int((self.digital_min * sample)/self.physical_min)

    #def __init__(self,label,file,header):
    def __init__(self,csv_files,header):
        line = header.readline().strip()
        self.filepostscript = self.get_field(header,line,'<filepostscript>')
        found = False
        for csv_file in csv_files:
            csv_split = csv_file.split('_')
            for element in csv_split:
                if element == self.filepostscript:
                    file = csv_file
                    found = True
                    break
            if found == True:
                break
        else:
            if found == False:
                print('Error: File corresponding to <filepostscript> label not detected')
                exit(1)
        self.label = self.get_field(header,line,'<label>').ljust(16)

        self.transducer = str(self.get_field(header,line,'<transducer>')).ljust(80)
        self.row = int(self.get_field(header,line,'<row>'))    #assign start row so data can be imported
        self.col = int(self.get_field(header,line,'<col>'))    #assign start col so data can be imported
        self.units = str(self.get_field(header,line,'<units>')).ljust(8)    #assign units
        self.physical_min = float(self.get_field(header,line,'<physical_min>'))    #get phys minimum
        self.physical_max = float(self.get_field(header,line,'<physical_max>'))    #get phys maximum
        self.digital_min = int(self.get_field(header,line,'<digital_min>'))      #get digital minimum
        self.digital_max = int(self.get_field(header,line,'<digital_max>'))      #get digital maximum
        self.prefiltering = str(self.get_field(header,line,'<prefiltering>')).ljust(80)    #get prefiltering info

        #header.close()  #close the header, we don't need it anymore
        
        self.time, self.samples = [],[] #declare arrays for time and samples
        with open('edf\\'+file,'r') as f:
            line = csv.reader(f)
            for element in line:
                try:
                #y = input(element[0])
                    self.time.append(float(element[0]))
                    self.scale_sample(float(element[self.col]))
                    self.samples.append(self.scaled_sample)
                except:
                    pass
        self.fs = int(1/self.time[1])   #get sampling freq/samples per record (each record spans 1 second)
        
        self.sub = [] #array to hold vars for writing this signal's subheader info
        self.sub.append(self.label)
        self.sub.append(self.transducer)
        self.sub.append(self.units)
        self.sub.append(str(self.physical_min).ljust(8))
        self.sub.append(str(self.physical_max).ljust(8))
        self.sub.append(str(self.digital_min).ljust(8))
        self.sub.append(str(self.digital_max).ljust(8))
        self.sub.append(str(self.prefiltering).ljust(80))
        self.sub.append(str(self.fs).ljust(8))
        self.sub.append(''.ljust(32)) #reserved 32 chars

        #iterate global var signal_count
        global signal_count
        signal_count += 1
        print('Signal detected: ', self.label)

class EDF:
    'class for creating the EDF file'

    def __init__(self):
        try:
            self.header = open('header.txt','r')
        except:
            print('Error: header.txt DNE. Is it in the Python directory?')
            exit(1)
        self.get_signals()
        line = self.header.readline().strip()
        #while line != '<output_edf>':
        #    line = self.header.readline().strip()
        #else:
        #    line = self.header.readline().strip()
        #self.output_name = line
        while line != '<patient_id>':
            line = self.header.readline().strip()
        line = self.header.readline().strip()
        self.patient_id = line.ljust(80)

        self.get_time()

        global signal_count
        
        #create string corresponding to top of header ("master header")
        self.master_header = ( '0'.ljust(8) + self.patient_id +
                               ('Startdate: ' + str(self.master_date_label)).ljust(80) +
                               str(self.start_date).ljust(8) + str(self.start_time).ljust(8) +
                               str((signal_count+1)*256).ljust(8) +
                               ''.ljust(44) + str(self.record_num).ljust(8) + '1'.ljust(8) +
                               str(signal_count).ljust(4) )

        #create subheader
        self.sub_header = ''
        for j in range(0,10):
            global signal_count
            for i in range(0,signal_count):
                self.sub_header += self.signals[i].sub[j]

        #write header
        print('Writing EDF file to: ', self.output_name)
        output = open(self.output_name,'wb')
        output.write(bytes(self.master_header,'utf-8'))
        output.write(bytes(self.sub_header,'utf-8'))

        #write samples
        exception = struct.pack('h',0)
        for i in range(0,self.record_num):
            for j in range(0,signal_count):
                for k in range((self.signals[j].fs * i),((self.signals[j].fs * i)+self.signals[j].fs)):
                    try:
                        output.write(struct.pack('h',self.signals[j].samples[k]))
                    except:
                        output.write(exception)
        print('File written successfully')
            
                
        
    def get_time(self):
        filename = self.csv_files[0].split('_')
        if filename[3] == '01':
            month = 'JAN'
        elif filename[3] == '02':
            month = 'FEB'
        elif filename[3] == '03':
            month = 'MAR'
        elif filename[3] == '04':
            month = 'APR'
        elif filename[3] == '05':
            month = 'MAY'
        elif filename[3] == '06':
            month = 'JUN'
        elif filename[3] == '07':
            month = 'JUL'
        elif filename[3] == '08':
            month = 'AUG'
        elif filename[3] == '09':
            month = 'SEP'
        elif filename[3] == '10':
            month = 'OCT'
        elif filename[3] == '11':
            month = 'NOV'
        elif filename[3] == '12':
            month = 'DEC'

        year = int(filename[2])
        self.master_date_label = filename[4] + '-' + month + '-' + filename[2]
        #Euro date convention for edf header
        self.start_date = filename[4] + '.' + filename[3] + '.' + str(year - round(year,-3))
        self.start_time = filename[6] + '.' + filename[7] + '.' + filename[8]

        #US date convetion for output file name
        self.output_name = 'stug_'+filename[3]+'_'+filename[4]+'_'+filename[2]+'__'+filename[6]+'_'+filename[7]+'_'+filename[8]+'.edf'
        
        
    def get_signals(self): #this function will need to be modified for each specific application
        self.signals = []
        self.signal_num = 0 #number of signals found
        #expected_signal_num = 12 #number of expected signals
        expected_signal_num = 2 #number of expected signals
        line = self.header.readline().strip()
        while line != '<csv_path>':
            line = self.header.readline().strip()
        line = self.header.readline().strip()
        path = line
        self.csv_files = []
        
        #creep through '~/edf' directory
        for root,dirs,files in os.walk(path):
            for file in files:
                if file.endswith('.csv'):
                    print('File detected: ',file)
                    self.csv_files.append(file)
        self.header.seek(0)
        prev_line_num = -1
        self.header.seek(1)
        line = self.header.readline().strip()
        curr_line_num = self.header.tell()
        while line != '</edf>':
            while (line != '<signal>'):
                line = self.header.readline().strip()
                self.signal_num += 1
                if line == '</edf>':
                    break
            else:
                self.signals.append(Signal(self.csv_files,self.header))
                try:
                    line = self.header.readline().strip()
                except:
                    pass

        #to figure out the number of records
        self.record_num = 0
        for sig in self.signals:
            length = (len(sig.samples)/sig.fs)+1
            if length > self.record_num:
                self.record_num = int(length)

        #rewind the file so it can be used by the EDF intializer       
        self.header.seek(0)
        #return signals,record_num

def main():
    edf = EDF()

main()
